<?php

namespace Drupal\copy_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media\Entity\Media;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Plugin implementation of the 'copy_media_embed_code_formatter' field formatter.
 *
 * @FieldFormatter(
 *   id = "copy_media_embed_code_formatter",
 *   label = @Translation("Copy"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class CopyMediaEmbedCodeFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    $target_entity_type = $field_definition->getFieldStorageDefinition()->getSetting('target_type');
    return $target_entity_type === 'media';
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      $entity = $item->entity;
      if ($entity instanceof Media) {
        $embed_code = '';
        $media_type = $entity->bundle();
        if ($media_type === 'image') {
          $file = $entity->field_media_image->entity;
          if ($file) {
            $path = $file->getFileUri();
            $image_path = \Drupal::service('file_url_generator')->generateAbsoluteString($path);
            $alt_text = $file->alt ?: '';
            $embed_code = '<img src="' . $image_path . '" alt="' . $alt_text . '">';
          }
        }
        elseif ($media_type === 'video') {
          $file = $entity->field_media_video_file->entity;
          if ($file) {
            $path = $file->getFileUri();
            $video_path = \Drupal::service('file_url_generator')->generateAbsoluteString($path);
            $embed_code = '<video src="' . $video_path . '"></video>';
          }
        }
        elseif ($media_type === 'audio') {
          $file = $entity->field_media_audio_file->entity;
          if ($file) {
            $path = $file->getFileUri();
            $audio_path = \Drupal::service('file_url_generator')->generateAbsoluteString($path);
            $embed_code = '<audio src="' . $audio_path . '"></audio>';
          }
        }
        elseif ($media_type === 'pdf') {
          $file = $entity->field_media_document->entity;
          if ($file) {
            $path = $file->getFileUri();
            $doc_path = \Drupal::service('file_url_generator')->generateAbsoluteString($path);
            $embed_code = '<iframe src="' . $doc_path . '"></iframe>';
          }
        }
        elseif ($media_type === 'remote_video') {
          $remote_video_url = $entity->field_media_oembed_video->value;
          if (!empty($remote_video_url)) {
            $embed_code = '<iframe src="' . $remote_video_url . '"></iframe>';
          }
        }
        $elements[$delta] = [
          '#type' => 'button',
          '#value' => $this->t('Copy'),
          '#attached' => [
            'library' => ['copy_formatter/copy_formatter'],
          ],
          '#attributes' => [
            'class' => ['copy-formatter'],
            'data-content' => $embed_code,
          ],
        ];
      }
      else {
        $elements[$delta] = $item->view();
      }
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = [];
    return $element;
  }

}
